/*
 * Copyright 2010 Semafor Informatik & Energie AG, Basel, Switzerland
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.

 */
package ch.semafor.gendas.dao;

import java.util.List;
import java.util.Map;

import ch.semafor.gendas.model.Element;
import ch.semafor.gendas.model.PropertyValue;

public interface ElementDao extends GenericDao<Element, Long> {
  List<Element> findByType(final String type);

  /**
   * get list of elements of given type and referenced element
   * @param type to search
   * @param ref element that is referenced
   * @return list of elements 
   */
  List<Element> findByTypeAndReference(final String type,
      final Element ref);

  /**
   * get list of property values with given type and value
   * @param ptype property type to search
   * @param value 
   * @return list of property types
   */
  List<PropertyValue> findPropertyTypeAndStringLike(
      final String ptype, final String value);

  /**
   * get list of elements with given type that have
   * property values of given property type and value
   * @param etype element type
   * @param ptype property type
   * @param value 
   * @return list of elements
   */
  List<Element> findByTypeAndPropertyTypeAndStringLike(
      final String etype, final String ptype, final String value);

  int getCountByTypeAndPropertyTypeAndStringLike(
      final String etype, final String ptype, final String value);
  
  /**
   * find elements of given type and properties 
   * @param etype
   * @param args map of key-value pairs
   * @param childargs map of child components with key-value pairs
   * @return list of found elements
   */
  List<Element> findElementsByArgs(final String etype, final Map<String, String>args
		  , final Map<String, Map<String, String>>childargs) ;
  
  /**
   * count elements by search filter
   * @param canonicalName
   * @param prop1
   * @param value1
   * @param prop2
   * @param value2
   * @return number of elements
   */
  int getCountByTypeAnd2PropertyTypesAndStringsLike(
      String canonicalName, String prop1, String value1, String prop2,
      String value2);

  /**
   * gat all active (not deleted) elements
   * @return list of active elements
   */
	List<Element> getAllActive();
}
