/*
 * Copyright 2010 Semafor Informatik & Energie AG, Basel, Switzerland
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.

 */
package ch.semafor.gendas.model;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.Index;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Entity
@Table(name = "property_values")
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
// @org.hibernate.annotations.Table(appliesTo="property_values", indexes = {
// @Index(name="idx", columnNames = { "svalue" } ) } )
public class PropertyValue implements Serializable {
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	// NOPMD by wim on 9/20/10 2:28 PM
	Long id; // NOPMD by wim on 9/20/10 2:25 PM

	@ManyToOne
	private PropertyValueList valuelist; // NOPMD by wim on 9/20/10 2:25 PM

	// Values
	@Index(name = "value_indx")
	private String svalue; // NOPMD by wim on 9/20/10 2:25 PM
	private Double dvalue; // NOPMD by wim on 9/20/10 2:25 PM
	private Integer ivalue; // NOPMD by wim on 9/20/10 2:25 PM
	private Long lvalue; // NOPMD by wim on 9/20/10 2:30 PM
	private Date date;
	private BigDecimal decimalValue;
	private Short bool;

	private static final long serialVersionUID = 1L;

	// Constructor
	public PropertyValue() {
		super();
	}

	// Constructor
	public PropertyValue(final PropertyValueList vlist) {
		super();
		this.valuelist = vlist;
	}

	// Copy-Constructor
	public PropertyValue(final PropertyValueList vlist, final PropertyValue value) {
		super();
		this.valuelist = vlist;
		this.svalue = value.svalue;
		this.dvalue = value.dvalue;
		this.ivalue = value.ivalue;
		this.lvalue = value.lvalue;
		this.date = value.date;
		this.bool = value.bool;
		this.decimalValue = value.decimalValue;
	}

	// Constructor
	public PropertyValue(final PropertyValueList vlist, final String value) {
		this.svalue = value;
		this.valuelist = vlist;
	}

	// Constructor
	public PropertyValue(final PropertyValueList vlist, final Double value) {
		this.dvalue = value;
		this.valuelist = vlist;
	}

	// Constructor
	public PropertyValue(final PropertyValueList vlist, final Integer value) {
		this.ivalue = value;
		this.valuelist = vlist;
	}

	// Constructor
	public PropertyValue(final PropertyValueList vlist, final Long value) {
		this.lvalue = value;
		this.valuelist = vlist;
	}

	public PropertyValue(final PropertyValueList vlist, final Boolean val) {
		// ToDo, PMD: AvoidUsingShortType
		this.bool = (short) (val ? 1 : 0);
		this.valuelist = vlist;
	}

	public PropertyValue(final PropertyValueList vlist, final BigDecimal val) {
		this.decimalValue = val;
		this.valuelist = vlist;
	}

	// Constructor
	public PropertyValue(final PropertyValueList vlist,
			final GregorianCalendar value) {
		setDate(value);
		this.valuelist = vlist;
	}

	public void setValueList(final PropertyValueList vlist) {
		this.valuelist = vlist;
	}

	public Element getElement() {
		return this.valuelist.getProperty().getElement();
	}

	public ElementType getElementType() {
		return this.valuelist.getProperty().getElement().getElementType();
	}

	public String getElementTypeName() {
		return this.valuelist.getProperty().getElement().getElementType().getName();
	}

	// getter and setter of String
	public String getString() {
		return svalue;
	}

	public void setString(final String value) {
		this.svalue = value;
	}

	// getter and setter of Double
	public Double getDouble() {
		return dvalue;
	}

	public void setDouble(final Double dvalue) {
		this.dvalue = dvalue;
	}

	// getter and setter of Boolean
	public Boolean getBool() {
		return this.bool == 1;
	}

	public void setBool(final Boolean bool) {
		// ToDo, PMD: AvoidUsingShortType
		this.bool = (short) (bool ? 1 : 0);
	}

	// getter and setter of Integer
	public Integer getInt() {
		return ivalue;
	}

	public void setInt(final Integer ivalue) {
		this.ivalue = ivalue;
	}

	// getter and setter of Long
	public Long getLong() {
		return lvalue;
	}

	public void setLong(final Long lvalue) {
		this.lvalue = lvalue;
	}

	// getter and setter of BigDecimal
	public BigDecimal getDecimal() {
		return decimalValue;
	}

	public void setDecimal(final BigDecimal bvalue) {
		this.decimalValue = bvalue;
	}

	// getter and setter of Date
	public GregorianCalendar getDate() {
		GregorianCalendar c = new GregorianCalendar(); // NOPMD by wim on 9/20/10
																										// 2:24 PM
		if (date != null)
			c.setTime(date);
		return c;
	}

	public void setDate(final GregorianCalendar date) {
		if (date == null) {
			return;
		}
		this.date = date.getTime();
	}

	public void setValue(final PropertyValue value) {
		this.svalue = value.svalue;
		this.dvalue = value.dvalue;
		this.ivalue = value.ivalue;
		this.lvalue = value.lvalue;
		this.date = value.date;
		this.bool = value.bool;
		this.decimalValue = value.decimalValue;
	}

	public boolean isValid() {
		if (isNull()) {
			return false; // NOPMD by wim on 9/20/10 2:23 PM
		}
		if (this.ivalue != null) {
			return true; // NOPMD by wim on 9/20/10 2:23 PM
		}
		if (this.lvalue != null) {
			return true; // NOPMD by wim on 9/20/10 2:23 PM
		}
		if (this.svalue != null) {
			if (this.svalue.length() > 0) { // NOPMD by wim on 9/20/10 2:27 PM
				return true; // NOPMD by wim on 9/20/10 2:29 PM
			}
		}
		if (this.dvalue != null) {
			return true; // NOPMD by wim on 9/20/10 2:28 PM
		}
		if (this.bool != null) {
			return true; // NOPMD by wim on 9/20/10 2:28 PM
		}
		if (this.decimalValue != null) {
			return true; // NOPMD by wim on 9/20/10 2:28 PM
		}
		if (this.date != null) {
			try {
				final XMLGregorianCalendar date = DatatypeFactory.newInstance()
						.newXMLGregorianCalendar();
				if (!this.date.equals(date.toGregorianCalendar().getTime())) {
					return true; // NOPMD by wim on 9/20/10 2:28 PM
				}
			} catch (DatatypeConfigurationException e) {
				return false; // fatal // NOPMD by wim on 9/20/10 2:28 PM
			}
		}
		return false;
	}

	public boolean isNull() {
		return this.ivalue == null && this.lvalue == null && this.svalue == null
				&& this.date == null && this.dvalue == null && this.bool == null
				&& this.decimalValue == null;
	}

	public String toString() {
		final ToStringBuilder sb = new ToStringBuilder(this,
				ToStringStyle.SHORT_PREFIX_STYLE); // NOPMD by wim on 9/20/10 2:25 PM
		sb.append("id", this.id);
		if (svalue != null) { // NOPMD by wim on 9/20/10 2:23 PM
			sb.append("String", svalue);
		} else if (dvalue != null) { // NOPMD by wim on 9/20/10 2:23 PM
			sb.append("Double", dvalue);
		} else if (ivalue != null) { // NOPMD by wim on 9/20/10 2:23 PM
			sb.append("Int", ivalue);
		} else if (lvalue != null) { // NOPMD by wim on 9/20/10 2:23 PM
			sb.append("Long", lvalue);
		} else if (date != null) { // NOPMD by wim on 9/20/10 2:23 PM
			sb.append("Date", date);
		} else if (bool != null) { // NOPMD by wim on 9/20/10 2:29 PM
			sb.append("Bool", bool);
		} else if (decimalValue != null) { // NOPMD by wim on 9/20/10 2:29 PM
			sb.append("BigDecimal", decimalValue);
		} else {
			sb.append("null");
		}
		return sb.toString();
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean equals(Object o) { // NOPMD by wim on 9/20/10 2:25 PM
		if (this == o) {
			return true; // NOPMD by wim on 9/20/10 2:31 PM
		}
		if (!(o instanceof PropertyValue)) {
			return false; // NOPMD by wim on 9/20/10 2:31 PM
		}

		final PropertyValue p = (PropertyValue) o; // NOPMD by wim on 9/20/10 2:30
																								// PM
		if (svalue == null && p.svalue != null) // NOPMD by wim on 9/20/10 2:24 PM
			return false; // NOPMD by wim on 9/20/10 2:31 PM
		if (dvalue == null && p.dvalue != null) // NOPMD by wim on 9/20/10 2:24 PM
			return false; // NOPMD by wim on 9/20/10 2:31 PM
		if (ivalue == null && p.ivalue != null) // NOPMD by wim on 9/20/10 2:24 PM
			return false; // NOPMD by wim on 9/20/10 2:31 PM
		if (lvalue == null && p.lvalue != null) // NOPMD by wim on 9/20/10 2:24 PM
			return false; // NOPMD by wim on 9/20/10 2:33 PM
		if (date == null && p.date != null) // NOPMD by wim on 9/20/10 2:24 PM
			return false; // NOPMD by wim on 9/20/10 2:33 PM
		if (bool == null && p.bool != null) // NOPMD by wim on 9/20/10 2:30 PM
			return false; // NOPMD by wim on 9/20/10 2:33 PM
		if (svalue != null) {
			if (p.svalue == null) // NOPMD by wim on 9/20/10 2:29 PM
				return false; // NOPMD by wim on 9/20/10 2:33 PM
			return p.svalue.equals(svalue); // NOPMD by wim on 9/20/10 2:33 PM
		}
		if (dvalue != null) {
			if (p.dvalue == null) // NOPMD by wim on 9/20/10 2:30 PM
				return false; // NOPMD by wim on 9/20/10 2:34 PM
			return p.dvalue.equals(dvalue); // NOPMD by wim on 9/20/10 2:35 PM
		}
		if (ivalue != null) {
			if (p.ivalue == null) // NOPMD by wim on 9/20/10 2:30 PM
				return false; // NOPMD by wim on 9/20/10 2:35 PM
			return p.ivalue.equals(ivalue); // NOPMD by wim on 9/20/10 2:35 PM
		}
		if (lvalue != null) {
			if (p.lvalue == null) // NOPMD by wim on 9/20/10 2:29 PM
				return false; // NOPMD by wim on 9/20/10 2:35 PM
			return p.lvalue.equals(lvalue); // NOPMD by wim on 9/20/10 2:35 PM
		}
		if (date != null) {
			if (p.date == null) // NOPMD by wim on 9/20/10 2:32 PM
				return false; // NOPMD by wim on 9/20/10 2:35 PM
			return p.date.equals(date); // NOPMD by wim on 9/20/10 2:35 PM
		}
		if (bool != null) {
			if (p.bool == null) // NOPMD by wim on 9/20/10 2:32 PM
				return false; // NOPMD by wim on 9/20/10 2:35 PM
			return p.bool.equals(bool); // NOPMD by wim on 9/20/10 2:35 PM
		}
		if (decimalValue != null) {
			if (p.decimalValue == null) // NOPMD by wim on 9/20/10 2:32 PM
				return false; // NOPMD by wim on 9/20/10 2:35 PM
			return p.decimalValue.equals(decimalValue); // NOPMD by wim on 9/20/10
																									// 2:35 PM
		}
		return true;
	}

	/**
	 * {@inheritDoc}
	 */
	public int hashCode() {
		final int prime = 31; // NOPMD by wim on 9/20/10 2:29 PM
		int result = 1;
		result = prime * result + (svalue != null ? svalue.hashCode() : 0); // NOPMD
																																				// by
																																				// wim
																																				// on
																																				// 9/20/10
																																				// 2:29
																																				// PM
		result = prime * result + (dvalue != null ? dvalue.hashCode() : 0); // NOPMD
																																				// by
																																				// wim
																																				// on
																																				// 9/20/10
																																				// 2:29
																																				// PM
		result = prime * result + (ivalue != null ? ivalue.hashCode() : 0); // NOPMD
																																				// by
																																				// wim
																																				// on
																																				// 9/20/10
																																				// 2:29
																																				// PM
		result = prime * result + (lvalue != null ? lvalue.hashCode() : 0); // NOPMD
																																				// by
																																				// wim
																																				// on
																																				// 9/20/10
																																				// 2:29
																																				// PM
		result = prime * result + (date != null ? date.hashCode() : 0); // NOPMD by
																																		// wim on
																																		// 9/20/10
																																		// 2:31 PM
		result = prime * result
				+ (decimalValue != null ? decimalValue.hashCode() : 0); // NOPMD by wim
																																// on 9/20/10
																																// 2:31 PM
		result = prime * result + (bool != null ? bool.hashCode() : 0); // NOPMD by
																																		// wim on
																																		// 9/20/10
																																		// 2:31 PM
		return result;
	}

	public Object getObject() {
		if (this.ivalue != null) {
			return this.ivalue; // NOPMD by wim on 9/20/10 2:23 PM
		}
		if (this.lvalue != null) {
			return this.lvalue; // NOPMD by wim on 9/20/10 2:23 PM
		}
		if (this.svalue != null) {
			if (this.svalue.length() > 0) { // NOPMD by wim on 9/20/10 2:27 PM
				return this.svalue; // NOPMD by wim on 9/20/10 2:29 PM
			}
		}
		if (this.dvalue != null) {
			return this.dvalue; // NOPMD by wim on 9/20/10 2:28 PM
		}
		if (this.bool != null) {
			return this.bool; // NOPMD by wim on 9/20/10 2:28 PM
		}
		if (this.decimalValue != null) {
			return this.decimalValue; // NOPMD by wim on 9/20/10 2:28 PM
		}
		if (this.date != null) {
			try {
				final XMLGregorianCalendar date = DatatypeFactory.newInstance()
						.newXMLGregorianCalendar();
				if (!this.date.equals(date.toGregorianCalendar().getTime())) {
					return date; // NOPMD by wim on 9/20/10 2:28 PM
				}
			} catch (DatatypeConfigurationException e) {
				return null; // fatal // NOPMD by wim on 9/20/10 2:28 PM
			}
		}

		return null;
	}
}
