/*
 * Copyright 2010 Semafor Informatik & Energie AG, Basel, Switzerland
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.

 */
package ch.semafor.gendas.model;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.hibernate.annotations.Index;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Entity
@Table(name = "property_value_list")
//@org.hibernate.annotations.Table(appliesTo="property_value_list", indexes = { @Index(name="idx", columnNames = { "next_revision" } ) } )
public class PropertyValueList implements Serializable {
	private static final long serialVersionUID = 1L;
	@Transient
	private final Logger logger = LoggerFactory.getLogger(PropertyValueList.class); // NOPMD by wildi on 9/21/10 7:31 AM
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	private Long id; // NOPMD by wim on 9/20/10 2:40 PM

	@OneToMany(mappedBy = "valuelist", cascade = CascadeType.ALL )
//	@org.hibernate.annotations.Cascade(value = { org.hibernate.annotations.CascadeType.DELETE_ORPHAN })
	private List<PropertyValue> values = new ArrayList<PropertyValue>(); // NOPMD by wim on 9/20/10 2:40 PM

	
  @ManyToOne
	@JoinColumn(name = "property_id")
	private Property property;

	@Column(nullable = false)
	private Long revision;

	@Column(nullable = false)
	@Index(name = "next_rev_indx")
	private Long nextRevision;

	// Constructor
	public PropertyValueList() {
		super();
		//ToDo PMD: NullAssignment
		this.property = null;
	}

	// Constructor
	public PropertyValueList(final Property pr) throws CoreException { // NOPMD by wim on 9/20/10 2:40 PM
		super();
		//ToDo PMD: NullAssignment
		this.id = null;
		this.property = pr;
		this.revision = pr.getNewRevision();
		this.nextRevision = Modification.MaxRevision;
		this.property.addValueList(this);
	}

	// Constructor
	private PropertyValueList(final PropertyValueList vlist, final Property p ) throws CoreException { // NOPMD by wim on 9/20/10 2:40 PM
		super();
		//ToDo PMD: NullAssignment
		this.id = null;

		this.property = p;
		this.revision = this.property.getNewRevision();
		this.nextRevision = Modification.MaxRevision;
		vlist.setNextRevision(this.revision);
    copyValues( vlist );
		this.property.addValueList(this);
	}

	private void copyValues(final  PropertyValueList vlist ){
	  this.values = new ArrayList<PropertyValue>();
	  for (PropertyValue v : vlist.values) {
	    this.values.add(new PropertyValue(this, v)); // NOPMD by wim on 9/20/10 2:39 PM
	  }
  }

	public boolean isTransient() {
		return this.id == null;
	}

	public boolean isPersistent() {
		return this.id != null;
	}

	public boolean isInRevision( final long rev ){
		if (rev == Modification.MaxRevision) {
			return this.nextRevision == Modification.MaxRevision; // NOPMD by wim on 9/20/10 2:38 PM
		}
		return rev >= this.revision && rev < this.nextRevision;
	}

	public boolean isLastRevision() {
		return this.nextRevision == Modification.MaxRevision;
	}

	// getter and setter of Property
	public void setProperty(final Property property) {
		this.property = property;
	}

	public Property getProperty() {
		return this.property;
	}

	// getter and setter of Revision
	public void setRevision(final long rev) {
		this.revision = rev;
	}

	public Long getRevision() {
		return this.revision;
	}

	// getter and setter of NextRevision
	public void setNextRevision(final long rev) {
		this.nextRevision = rev;
	}

	public Long getNextRevision() {
		return this.nextRevision;
	}

	public boolean isEmpty() {
		return this.values.isEmpty();
	}

  public boolean isValid() {
    if ( this.values.isEmpty() ){
      return false; // NOPMD by wim on 9/20/10 2:38 PM
    }
    for (PropertyValue v : values) {
      if( v.isValid() ){
        return true; // NOPMD by wim on 9/20/10 2:38 PM
      }
    }
    return false;
  }

  public boolean isntValid() {
    return !isValid();
  }
  
	// getter of Values
	public String getString(final int index) {
		final PropertyValue v = values.get(index); // NOPMD by wim on 9/20/10 2:40 PM
		return v != null ? v.getString() : null; // NOPMD by wim on 9/20/10 2:39 PM
	}

	public Double getDouble(final int index) {
		final PropertyValue v = values.get(index); // NOPMD by wim on 9/20/10 2:40 PM
		return v != null ? v.getDouble() : null; // NOPMD by wim on 9/20/10 2:39 PM
	}

	public Integer getInt(final int index) {
		final PropertyValue v = values.get(index); // NOPMD by wim on 9/20/10 2:43 PM
		return v != null ? v.getInt() : null; // NOPMD by wim on 9/20/10 2:39 PM
	}

	public Long getLong(final int index) {
		final PropertyValue v = values.get(index); // NOPMD by wim on 9/20/10 2:43 PM
		return v != null ? v.getLong() : null; // NOPMD by wim on 9/20/10 2:39 PM
	}
  public Boolean getBool(final int index) {
	  final PropertyValue v = values.get(index); // NOPMD by wim on 9/20/10 2:43 PM
    //FB: NP_BOOLEAN_RETURN_NULL
    return v != null ? v.getBool() : null; // NOPMD by wim on 9/20/10 2:39 PM
  }
  public BigDecimal getDecimal(final int index) {
	  final PropertyValue v = values.get(index); // NOPMD by wim on 9/20/10 2:43 PM
    //FB: NP_BOOLEAN_RETURN_NULL
    return v != null ? v.getDecimal() : null; // NOPMD by wim on 9/20/10 2:39 PM
  }

	public GregorianCalendar getDate(final int index) {
		final PropertyValue v = values.get(index); // NOPMD by wim on 9/20/10 2:43 PM
		return v != null ? v.getDate() : null; // NOPMD by wim on 9/20/10 2:42 PM
	}

	// setter of Values
	public PropertyValueList setValue(final int index, final PropertyValue value)
			throws CoreException {
		PropertyValueList vlist = this;
		boolean valueChanged = false;

		if (isPersistent()) {
			if (vlist.values.size() <= index) {
				valueChanged = true;
			} else {
				final PropertyValue val = values.get(index);
				if (val == null) {
					valueChanged = true;
				} else {
					if (!val.equals(value)) {
						valueChanged = true;
					}
				}
			}
			if (!valueChanged) {
				return this; // NOPMD by wim on 9/20/10 2:38 PM
			}
			vlist = new PropertyValueList(this, this.property);
		}
		if (vlist.values.size() < index + 1) {
			final PropertyValue new_value = new PropertyValue(vlist, value);
			vlist.values.add(index, new_value);
		} else {
			final PropertyValue cur_value = vlist.values.get(index);
			if (cur_value == null) {
				throw new CoreException("FATAL: Expected Value not available");
			}
			cur_value.setValue(value);
		}
		return vlist;
	}

	public PropertyValueList setString(final int index, final String val)
			throws CoreException {
		final PropertyValue value = new PropertyValue(null, val);
		return setValue(index, value);
	}

	public PropertyValueList setDouble(final int index, final Double val)
			throws CoreException {
		final PropertyValue value = new PropertyValue(null, val);
		return setValue(index, value);
	}

	public PropertyValueList setInt(final int index, final Integer val)
			throws CoreException {
		final PropertyValue value = new PropertyValue(null, val);
		return setValue(index, value);
	}

	public PropertyValueList setLong(final int index, final Long val) throws CoreException {
		PropertyValue value = new PropertyValue(null, val); // NOPMD by wim on 9/20/10 2:45 PM
		return setValue(index, value);
	}

	public PropertyValueList setDecimal(final int index, final BigDecimal val) throws CoreException {
		PropertyValue value = new PropertyValue(null, val); // NOPMD by wim on 9/20/10 2:45 PM
		return setValue(index, value);
	}

	public PropertyValueList setDate(final int index, final GregorianCalendar val)
	       throws CoreException {
		PropertyValue value = new PropertyValue(null, val); // NOPMD by wim on 9/20/10 2:45 PM
		return setValue(index, value);
	}

	// getter and setter of Id
	public Long getId() {
		return id;
	}

	public void setId(Long id) { // NOPMD by wim on 9/20/10 2:43 PM
		this.id = id;
	}

	public String toString() {
		ToStringBuilder sb = new ToStringBuilder(this,ToStringStyle.SHORT_PREFIX_STYLE); // NOPMD by wim on 9/20/10 2:45 PM
		sb.append("id", this.id);
		sb.append("revision", revision + " -> " + nextRevision);
		sb.append("size", this.values.size());
		for (PropertyValue v : values) {
			sb.append(v.toString());
		}
		return sb.toString();
	}

	private void indent(final int ind) {
		property.getElement().indent(ind);
	}

	public void print(final int ind) {
    ToStringBuilder sb = new ToStringBuilder(this,ToStringStyle.SHORT_PREFIX_STYLE); // NOPMD by wim on 9/20/10 2:45 PM
	  sb.append("id", this.id);
	  sb.append("revision", revision + "-" + nextRevision);
	  sb.append("size", this.values.size());
		indent(ind);
		System.out.println(sb.toString()); // NOPMD by wim on 9/20/10 2:42 PM
		if (this.values.size() > 0) {
			indent(ind + 1);
			System.out.print("("); // NOPMD by wim on 9/20/10 2:42 PM
			for (PropertyValue v : this.values) {
				System.out.print(" "); // NOPMD by wim on 9/20/10 2:42 PM
				System.out.print(v.toString()); // NOPMD by wim on 9/20/10 2:42 PM
			}
			System.out.println(" )"); // NOPMD by wim on 9/20/10 2:42 PM
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean equals(Object o) { // NOPMD by wim on 9/20/10 2:45 PM
		if (this == o) {
			// logger.debug("identical");
			return true; // NOPMD by wim on 9/20/10 2:38 PM
		}
		if (!(o instanceof PropertyValueList)) {
			// logger.debug("different class");
			return false; // NOPMD by wim on 9/20/10 2:42 PM
		}
		final PropertyValueList vlist = (PropertyValueList) o;
		if (this.values.size() != vlist.values.size()) {
			return false; // NOPMD by wim on 9/20/10 2:42 PM
		}
		if (this.values.size() == 0) {
			return true; // NOPMD by wim on 9/20/10 2:42 PM
		}

		Iterator<PropertyValue> v1 = this.values.iterator(); // NOPMD by wim on 9/20/10 2:45 PM
		Iterator<PropertyValue> v2 = vlist.values.iterator(); // NOPMD by wim on 9/20/10 2:45 PM
		while (v1.hasNext()) {
			final PropertyValue pv1 = v1.next();
			final PropertyValue pv2 = v2.next();
			if (pv1 == null) {
				if (pv2 != null && !pv2.isNull()) {
					return false; // NOPMD by wim on 9/20/10 2:42 PM
				}
			} else if (pv2 == null) {
				if (!pv1.isNull()) {
					return false; // NOPMD by wim on 9/20/10 2:42 PM
				}
			} else if (!pv1.equals(pv2)) {
				// logger.debug("different values");
				return false; // NOPMD by wim on 9/20/10 2:45 PM
			}
		}
		// logger.debug("equal values: size "+values.size()
		// +"/"+p.values.size()+" hashcode " + values.hashCode() + "/" +
		// p.values.hashCode() );

		return true;
	}

	/**
	 * {@inheritDoc}
	 */
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (property != null ? property.hashCode() : 0); // NOPMD by wim on 9/20/10 2:42 PM
		result = prime * result + (values != null ? values.hashCode() : 0); // NOPMD by wim on 9/20/10 2:45 PM
		return result;
	}

	public void assign(final PropertyValueList src_vlist) throws CoreException {
	  logger.debug("begin of PropertyValueList::assign()");
    
	  PropertyValueList dest_vlist = this;
	  if( isTransient() ){
      if( !isEmpty() ){ // NOPMD by wim on 9/20/10 2:44 PM
        logger.debug("--> transient valuelist is NOT empty ?!");
      }	    
	  }
    if( src_vlist.isEmpty()){
      throw new CoreException("a source-valuelist must have values");
    }

		if( src_vlist.values.size() != dest_vlist.values.size() ){ // NOPMD by wim on 9/20/10 2:42 PM
		  // The values are modified. We need to create a
		  // new container
		  if( dest_vlist.isTransient()){
		    logger.debug("--> update of transient valuelist");
		  }
		  else{
		    dest_vlist.setNextRevision( this.property.getNewRevision() );
        dest_vlist = new PropertyValueList( this.property );
        logger.debug("--> new valuelist created");
		  }
		  dest_vlist.copyValues( src_vlist );
		}
		else{
      for( int i=0; i < src_vlist.values.size(); i++ ){
        // Wenn der Wert geaendert hat, wird unter Umstaenden eine neue
        // ValueList erstellt und die Alte wird 'historisch'.
        dest_vlist = dest_vlist.setValue(i, src_vlist.values.get( i ) );
      }
      if( dest_vlist != this ){
        logger.debug("--> values updated in valuelist");
      }

		}
		
    if( dest_vlist.values.size() == 0 ){ // NOPMD this is JDK 1.5 by wim on 9/20/10 2:41 PM
      logger.debug("--> error: a valuelist must have values: Element: " + getProperty().getElement().toString() );
      throw new CoreException("a valuelist must have values");
    }
    logger.debug("end of PropertyValueList::assign()");
	}
	public List<PropertyValue> getValues() {
    return this.values;
  }
	
}
