package ch.semafor.gendas.service;
// http://www.gridshore.nl/2008/04/29/using-ehcache-and-verifying-that-it-works-with-jpa-and-springframework/
import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.semafor.gendas.model.Element;
import ch.semafor.gendas.model.ElementRefList;
import ch.semafor.gendas.model.ElementRefs;

import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheManager;

public class PersistenceCache {
	private Map<Long, Long> parentMap;
	private Cache cache = null;
	final Logger logger = LoggerFactory.getLogger(getClass());
	
	public PersistenceCache(){
		CacheManager cacheManager = CacheManager.create();
		logger.debug("get cache {} "+getClass().getName());
		this.cache = cacheManager.getCache(getClass().getName());
		this.cache.setStatisticsEnabled(true);
		this.parentMap = new HashMap<Long, Long>();
	}

	private String getKey( Long id, Long rev ){
		StringBuffer key = new StringBuffer(id.toString());
		if (rev != null) {
			key.append('.');
			key.append(rev.toString());
		}
		return key.toString();
	}
	
	private void setParentMap( Element e ){
		for( ElementRefs r: e.getListOfElementRefs() ){
			for( ElementRefList rl: r.getReflist() ){
				for( Element c: rl.getElementList() ){
					logger.debug(c.getId() + " --> " + e.getId());
					if( parentMap.put(c.getId(), e.getId())==null ){
						setParentMap( c );
					}
				}
			}
		}
	}
	
	public synchronized Object getCachedObject(Long id, Long rev) {
		if (cache != null) {
			final net.sf.ehcache.Element cacheElement = cache.get(getKey(id,rev));
			if (cacheElement != null) {
				logger.debug("found object {} in cache", getKey(id,rev));
				return cacheElement.getValue();
			}
			logger.info("missing object {} in cache", getKey(id,rev));				
		}
		return null;
	}

	public synchronized void putCachedObject(Element e, Long rev, Object obj) {
		if (cache != null) {
			logger.debug("put object {}", getKey(e.getId(),rev));
			final net.sf.ehcache.Element cacheElement = new net.sf.ehcache.Element(
					getKey(e.getId(),rev), obj);
			cache.put(cacheElement);
			setParentMap( e );
		}
	}

	public synchronized void remove( Long id, Long rev ){
		if (cache != null) {
			logger.debug("remove object {}", getKey(id, rev));
			Long parent=id;
			while( parent!=null ){
				cache.remove(getKey(id, rev));
				parent = parentMap.get(id);
				parentMap.remove(id);
				logger.debug(id + " --> " + parent);
				id = parent;
			}
		}
	}

	public net.sf.ehcache.Statistics getStatistics() {
		return cache.getStatistics();
	}
}
